module.exports = ({ config, paths }) => {
  const IS_MODE_PROD = process.env.APP_MODE === 'production'

  require('./babel.loader')({ config, paths })
  require('./style.loader')({ config, paths })
  require('./asset.loader')({ config, paths })

  config
    .mode(process.env.APP_MODE)
    .entry('main')
    .add(paths.appEntry)
    .end()
    .output.path(paths.appBuild)
    .filename(
      IS_MODE_PROD
        ? `${paths.assetsSubPath}${paths.assetsJsPath}[name].[contenthash:8].js`
        : '[name].js'
    )
    .publicPath(paths.publicPath)
    .end()
    .resolve.alias.set('@', paths.appSrc)
    .end()
    .extensions.merge(['.ts', '.tsx', '.js'])

  config
    .plugin('error')
    .use(require('friendly-errors-webpack-plugin'))
    .end()
    .plugin('env')
    .use(require('dotenv-webpack'), [
      {
        path: `.env.${process.env.APP_ENV}`
      }
    ])
    .end()
    .plugin('html')
    .use(require('html-webpack-plugin'), [
      {
        template: paths.appTemplate,
        filename: 'index.html'
      }
    ])
    .end()
    .plugin('copy')
    .use(require('copy-webpack-plugin'), [
      {
        patterns: [
          {
            from: paths.appStatic,
            to: paths.appBuild,
            globOptions: {
              ignore: ['index.html']
            }
          }
        ]
      }
    ])

  config.when(IS_MODE_PROD, (config) => {
    require('./optimization')({ config, paths })
  })

  config.when(!IS_MODE_PROD, (config) => {
    require('./dev-server')({ config, paths })
    config.devtool('cheap-module-eval-source-map')
  })
}
